﻿Imports System.IO
Imports System.Text.RegularExpressions

Public Class Form1

    ' Declare necessary variables
    Private searchResults As New List(Of SearchResult)()
    Private searchFolderPath As String = Application.StartupPath  ' Default to the application directory
    Private searchWords As String()
    Private isCancel As Boolean

    Private Const RecordsPerPage As Integer = 10

    Private Sub btnSelectFolder_Click(sender As Object, e As EventArgs) Handles btnSelectFolder.Click
        Using folderBrowser As New FolderBrowserDialog()
            folderBrowser.Description = "Select the folder to search"
            folderBrowser.RootFolder = Environment.SpecialFolder.MyComputer

            If folderBrowser.ShowDialog() = DialogResult.OK Then
                searchFolderPath = folderBrowser.SelectedPath
                TextBox1.Text = searchFolderPath
                ''MessageBox.Show("Selected folder: " & searchFolderPath, "Folder Selected", MessageBoxButtons.OK, MessageBoxIcon.Information)
            End If
        End Using
    End Sub

    Private Sub btnSearch_Click(sender As Object, e As EventArgs) Handles btnSearch.Click
        btnSearch.Enabled = False
        isCancel = False
        lstResults.Items.Clear()
        lblStatus.Text = ""
        searchResults.Clear()
        isCancel = False

        ' Read and process the search query
        Dim strSearchWords As String = txtSearch.Text.Trim()
        If String.IsNullOrEmpty(strSearchWords) Then
            MessageBox.Show("Please enter at least one keyword to search")
            Return
        End If

        ' Prepare search words
        If rbPhrase.Checked Then
            searchWords = New String() {strSearchWords}
        Else
            searchWords = strSearchWords.Split(" "c)
        End If

        ' Search the selected directory
        SearchFiles(New DirectoryInfo(searchFolderPath))

        ' Display results
        DisplayResults()
        btnSearch.Enabled = True
    End Sub

    Private Sub SearchFiles(folder As DirectoryInfo)
        Try
            ' Search through all files
            For Each file In folder.GetFiles()
                Application.DoEvents()
                If isCancel = True Then
                    lblStatus.Text = "Cancelled!"
                    Exit Sub
                End If
                If file.Extension.Equals(".htm") OrElse file.Extension.Equals(".html") _
                    OrElse file.Extension.Equals(".asp") OrElse file.Extension.Equals(".shtml") Then

                    Dim content As String = IO.File.ReadAllText(file.FullName)
                    Dim title = GetMetaTag(content, "<title>", "</title>")
                    Dim description = GetMetaTag(content, "<meta name=""description"" content=""", """>")
                    Dim description1 = GetMetaTag(content, "<meta name=""description"" content=""", """ />")

                    If description1 <> "" AndAlso description1.Length < description.Length Then
                        description = description1
                    End If

                    Dim matches As Integer = 0
                    For Each word In searchWords
                        Dim regex As New Regex("\b" & Regex.Escape(word) & "\b", RegexOptions.IgnoreCase)
                        Dim matchCollection = regex.Matches(content)
                        matches += matchCollection.Count
                    Next

                    If matches > 0 Then
                        searchResults.Add(New SearchResult With {
                            .Title = title,
                            .Path = file.FullName,
                            .Description = description,
                            .MatchesCount = matches
                        })
                    End If
                Else
                    Dim content As String = IO.File.ReadAllText(file.FullName)

                    Dim matches As Integer = 0
                    For Each word In searchWords
                        Dim regex As New Regex("\b" & Regex.Escape(word) & "\b", RegexOptions.IgnoreCase)
                        Dim matchCollection = regex.Matches(content)
                        matches += matchCollection.Count
                    Next

                    If matches > 0 Then
                        searchResults.Add(New SearchResult With {
                            .Title = "",
                            .Path = file.FullName,
                            .Description = "",
                            .MatchesCount = matches
                        })
                    End If
                End If
            Next

            ' Search subdirectories
            For Each subFolder In folder.GetDirectories()
                SearchFiles(subFolder)
            Next

        Catch ex As Exception
            MessageBox.Show("Error reading files: " & ex.Message)
        End Try
    End Sub

    Private Function GetMetaTag(content As String, startTag As String, endTag As String) As String
        Dim startIndex = content.IndexOf(startTag, StringComparison.OrdinalIgnoreCase)
        If startIndex < 0 Then Return ""

        startIndex += startTag.Length
        Dim endIndex = content.IndexOf(endTag, startIndex, StringComparison.OrdinalIgnoreCase)

        If endIndex < 0 Then Return ""

        Return content.Substring(startIndex, endIndex - startIndex).Trim()
    End Function

    Private Sub DisplayResults()
        If searchResults.Count = 0 Then
            lstResults.Items.Add("No results found.")
            Return
        End If

        For Each result In searchResults
            lstResults.Items.Add($"{result.Title} - {result.Path} (Matches: {result.MatchesCount})")
        Next

        lblStatus.Text = $"Searched {searchResults.Count} files."
    End Sub

    Private Sub Form1_Load(sender As Object, e As EventArgs) Handles Me.Load
        lstResults.HorizontalExtent = 2000
        KeyPreview = True
    End Sub

    Private Sub Form1_KeyDown(sender As Object, e As KeyEventArgs) Handles Me.KeyDown
        If e.KeyCode = Keys.Escape Then
            isCancel = True
        End If
    End Sub
End Class

Public Class SearchResult
    Public Property Title As String
    Public Property Path As String
    Public Property Description As String
    Public Property MatchesCount As Integer
End Class
